<?php
/**
 * File: $Id: datalist.php,v 1.15 2005/04/19 19:50:19 greyhead Exp $
 * Ciamos - Content Management System
 * This class is for handling table data
 * @package Ciamos
 * @copyright (C) 2004 Ciamos Development Team.
 * @license LGPL {@link http://www.gnu.org/licenses/lgpl.html}
 * @link http://www.ciamos.com
 * @subpackage Ciamos_xxx
 * @link link to information for the subpackage
 * @author Florian Solcher (alphalogic) (@link http://www.alphalogic.org)
 *
 */

Class DataList 
{
    var $node = false;
    var $fields;
    var $id_field;
    var $table;

    /**
     * Class constructor
     * @param array $resource a table schema array : array( 1 => array('field' => 'value'))
     * @return unknown
     */
    function DataList($resource = false, $table = 0, $id_field = 0)
    {
        if ( $resource ) {
            $me             = array_shift($resource);
            $this->table    = $table;
            $this->id_field = $id_field;
            foreach ( $me as $field => $value ) {
                $this->$field   = $value;
                $this->fields[] = $field;
            }
            if ( is_array($resource) ) {
                if ( count($resource) > 0 ) {
                    $this->node = new DataList($resource, $this->table, $this->id_field);
                }
            } else {
                $this->node = false;
            }

        }
    }
    
    /**
     * Checks if the list has more elements
     * @return bool
     */
    function hasMoreElements() 
    {
        return (boolean) is_object($this->node);
    }
    
    /**
     * Drop the first element of the list
     * @return bool
     */
    function nextElement() 
    {
        $return = false;
        if ( $this->node ) {
            foreach ( $this->fields as $field => $value ) {
                $this->$field = $this->node->$field;
            }
            $this->node = $this->node->node;
            $return = true;
        }
        return $return;
    }

    /**
     * Returns the element that has a given id
     * @param integer $t_id the id for the element
     * @return array
     */
    function getItem($asobject = true, $id='')
    {
        $field  = $this->id_field;
        if ( $this->$field == $id || !is_numeric($id) ) {
            if ( $asobject ) {
                return $this;
            } else {
                $array = array();
                foreach ($this->fields as $field) {
                    $array[$field] = $this->$field;
                }
                return $array;
            }
        } elseif ($this->node) {
            return $this->node->getItem($asobject, $id);
        } else {
            return false;
        }
    }

    /**
     * Returns an array of objects, note: NO class objects - use nextElement instead
     * @param integer $t_id the id for the element
     * @return Object
     */
    function listItems($asobject = false, $ret = array())
    {
        $field = $this->id_field;
        $ret[$this->$field] = $this->getItem($asobject);
        if ($this->node) {
            return $this->node->listItems($asobject, $ret);
        } else {
            return $ret;
        }

    }
    /**
     * Returns the max id of all elements
     * @return integer
     */
    function getMaxId($id = 0) 
    {
        $field = $this->id_field;
        if ( $this->$field > $id ) {
            $id = $this->$field;
        } else {
            return $id;
        }
        if ( $this->node ) {
            return $this->node->getMaxId($id);
        } else {
            return $id;
        }
    }

    /**
     * Returns the the first missing id
     * @return integer
     */
    function getMissingId()
    {
        $field = $this->id_field;
        if (!$this->node) return false;
        $parentID   = $this->$field;
        $childID    = $this->node->$field;
        $parentID++;
        if ($parentID != $childID) {
            return $parentID;
        } elseif($this->node->node) {
            $this->node->getMissingId();
        } else {
            return false;
        }
    }

    /**
     * Returns the best next id for the insert statement
     * @return integer
     */
    function getNextInsertId()
    {
        $missingID  = $this->getMissingId();
        $maxID      = $this->getMaxId();
        $maxID++;
        return ($missingI) ? $missingID : $maxID;
    }

    /**
     * Checks if a specfied $value for a specified $field exists in the list
     * @param string field
     * @param mixed value
     * @return bool
     */
    function exists($field, $value)
    {
        if ($this->$field == $value) {
            return true;
        } elseif ($this->node) {
            return $this->node->exists($field, $value);
        } else {
            return false;
        }
    }
    
    /**
     * Checks if a specfied $value for a specified $field exists in the list
     * @param string field
     * @param mixed value
     * @return bool
     */
    function findFirstItem($field, $value, $asobject = true)
    {
        if ( is_array($field) ) {
            $append = true;
            foreach ($field as $key => $f){
                if ($this->$f != $value[$key]) $append = false;
            }
            if ($append) {
                return $this->getItem($asobject);
            } elseif ($this->node) {
                return $this->node->findFirstItem($field, $value, $asobject);
            } else {
                return false;
            }
        } elseif (is_string($field)) {
            if ($this->$field == $value) {
                return $this;
            } elseif ($this->node) {
                return $this->node->findFirstItem($field, $value, $asobject);
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     * Enter description here...
     *
     * @param array/string $field
     * @param array $value
     * @param array $storage
     * @param boolean $asobject
     * @return unknown
     */
    function findItems($field, $value, $storage=array(), $asobject = false)
    {
        if ( is_array($field) ) {
            $append = true;
            foreach ($field as $key => $f){
                if ($this->$f != $value[$key]) $append = false;
            }
            if ( $append ) {
                $storage[] = $this->getItem($asobject);
                if ($this->node) {
                    return $this->node->findItems($field, $value, $storage, $asobject);
                } else {
                    return $storage;
                }
            } elseif ($this->node) {
                return $this->node->findItems($field, $value, $storage, $asobject);
            } else {
                return $storage;
            }
        } elseif ( is_string($field) ) {
            if ($this->$field == $value) {
                $storage[] = $this->getItem($asobject);
                if ($this->node) {
                    return $this->node->findItems($field, $value, $storage, $asobject);
                } else {
                    return $storage;
                }
            } elseif ($this->node) {
                return $this->node->findItems($field, $value, $storage, $asobject);
            } else {
                return $storage;
            }
        } else {
            return $storage;
        }
    }

    /**
     * Appends an element to the end of the List (Helper function)
     * @param array $add_array a table row schema array : array('field' => 'value')
     * @return bool
     */
    function addItem($add_array)
    {
        $id = $this->getMaxId();
        $id++;
        return $this->addItemWithId($add_array, $id);
    }

    /**
     * Appends an element to the end of the List
     * @param array $add_array a table row schema array : array('field' => 'value')
     * @param integer $t_id the id for the new row
     * @return bool
     */
    function addItemWithId($add_array, $id)
    {
        global $db;

        $add_array[$this->id_field]     = $id;
        foreach ($add_array as $field => $value) {
            $inserts[]  = $field." = '".$value."'";
        }
        $sql = '
            INSERT
                INTO '.$this->table.'
                SET '.join(', ', $inserts);
        echo $sql;
        die();
        if ( $this->hasMoreElements() ) {
            $this->node->addItemWithId($add_array, $id);
        } else {
            $db->query($sql);
            $x_array[] = $add_array;
            $this->node = new DataList($x_array, $this->table, $this->id_field);
            return $this->node;
        }
    }

    /**
     * Update the element where we are right now
     * @param array $add_array a table row schema array : array('field' => 'value')
     * @return bool
     */
    function update($update_array, $datalist_obj)
    {
        $field = $this->id_field;
        $bool = $datalist_obj->updateItem($update_array, $this->$field);
        foreach ($update_array as $field => $value) {
            $this->$field   = $value;
        }
        return $bool;
    }

    /**
     * Update the element that has a given id
     *
     * @param array $update_array a table row schema array : array('field' => 'value')
     * @param integer $t_id the id for the element
     * @return bool
     */
    function updateItem($update_array, $id)
    {
        global $db;

        foreach ($update_array as $field => $value) {
            $updates[]  = $field." = '".$value."'";
        }
        $sql = '
            UPDATE '.$this->table.'
                SET '.join(', ', $updates).'
                WHERE '.$this->id_field.'='.$id;
        $field = $this->id_field;
        if ($this->$field) {
            $db->query($sql);

            foreach ($update_array as $field => $value) {
                $this->$field   = $value;
            }
            return true;
        } elseif ($this->node) {
            $this->node->updateItem($update_array, $id);
        } else {
            return false;
        }
    }

    /**
     * Delete the element where we are right now
     * @return bool
     */
    function delete($datalist_obj)
    {
        $field = $this->id_field;
        $bool = $datalist_obj->deleteItem($this->$field);
        unset($this);
        return $bool;
    }

    /**
     * Delete the element that has a given id
     * @param integer $t_id the id for the element
     * @return bool
     */
    function deleteItem($id)
    {
        global $db;
        $sql = '
            DELETE
                FROM '.$this->table.'
                WHERE '.$this->id_field.'='.$id;
        $field          = $this->id_field;
        if ($this->node) {
            if ($this->node->$field== $id) {
                $this->node = $this->node->node;
                $db->query($sql);
                return true;
            } elseif ($this->$field== $id) {
                foreach ($this->fields as $field) {
                    $this->$field = $this->node->$field;
                }
                $this->node = $this->node->node;
                $db->query($sql);
                return true;
            } else {
                $this->node->deleteItem($id);
            }
        } elseif ($this->$field == $id) {
            unset($this);
            $db->query($sql);
            return true;
        } else {
            return false;
        }
    }
}

// Class End -->


/**
 * Reads from a database into a datalist - used to write caches
 *
 * @param string $sql sql query
 * @param string $table dbase table name
 * @param string $idfield selector field name
 * @return object new datalist
 */
function initDataList($sql, $table, $idfield)
{
    global $db;

    $result = $db->query($sql);
    $call_array = array();
    $i = 0;
    while ( $array   = $db->fetch_array($result) ) {
        foreach ( $array as $key => $value ) {
            if ( !is_numeric($key) ) {
                $call_array[$i][$key] = $value;
            }
        }
        $i++;
    }
    return new DataList($call_array, $table, $idfield);
}
?>